@@ -757,19 +757,17 @@ func capabilitiesFromConfig(pConf *config.Neighbor) []bgp.ParameterCapabilityInt
 		}
 	}
 
-	// unnumbered BGP
-	if pConf.Config.NeighborInterface != "" {
-		tuples := []*bgp.CapExtendedNexthopTuple{}
-		families, _ := config.AfiSafis(pConf.AfiSafis).ToRfList()
-		for _, family := range families {
-			if family == bgp.RF_IPv6_UC {
-				continue
-			}
-			tuple := bgp.NewCapExtendedNexthopTuple(family, bgp.AFI_IP6)
-			tuples = append(tuples, tuple)
+	// Extended Nexthop Capability (Code 5)
+	tuples := []*bgp.CapExtendedNexthopTuple{}
+	families, _ := config.AfiSafis(pConf.AfiSafis).ToRfList()
+	for _, family := range families {
+		if family == bgp.RF_IPv6_UC {
+			continue
 		}
-		caps = append(caps, bgp.NewCapExtendedNexthop(tuples))
+		tuple := bgp.NewCapExtendedNexthopTuple(family, bgp.AFI_IP6)
+		tuples = append(tuples, tuple)
 	}
+	caps = append(caps, bgp.NewCapExtendedNexthop(tuples))
 
 	// ADD-PATH Capability
 	if c := capAddPathFromConfig(pConf); c != nil {
@@ -1254,9 +1252,8 @@ func (h *fsmHandler) opensent(ctx context.Context) (bgp.FSMState, *fsmStateReaso
 				continue
 			}
 			e := i.(*fsmMsg)
-			switch e.MsgData.(type) {
+			switch m := e.MsgData.(type) {
 			case *bgp.BGPMessage:
-				m := e.MsgData.(*bgp.BGPMessage)
 				if m.Header.Type == bgp.BGP_MSG_OPEN {
 					fsm.lock.Lock()
 					fsm.recvOpen = m
@@ -1408,8 +1405,8 @@ func (h *fsmHandler) opensent(ctx context.Context) (bgp.FSMState, *fsmStateReaso
 					return bgp.BGP_FSM_IDLE, newfsmStateReason(fsmInvalidMsg, nil, nil)
 				}
 			case *bgp.MessageError:
-				m, _ := fsm.sendNotificationFromErrorMsg(e.MsgData.(*bgp.MessageError))
-				return bgp.BGP_FSM_IDLE, newfsmStateReason(fsmInvalidMsg, m, nil)
+				msg, _ := fsm.sendNotificationFromErrorMsg(m)
+				return bgp.BGP_FSM_IDLE, newfsmStateReason(fsmInvalidMsg, msg, nil)
 			default:
 				log.WithFields(log.Fields{
 					"Topic": "Peer",
@@ -1524,9 +1521,8 @@ func (h *fsmHandler) openconfirm(ctx context.Context) (bgp.FSMState, *fsmStateRe
 				continue
 			}
 			e := i.(*fsmMsg)
-			switch e.MsgData.(type) {
+			switch m := e.MsgData.(type) {
 			case *bgp.BGPMessage:
-				m := e.MsgData.(*bgp.BGPMessage)
 				if m.Header.Type == bgp.BGP_MSG_KEEPALIVE {
 					return bgp.BGP_FSM_ESTABLISHED, newfsmStateReason(fsmOpenMsgNegotiated, nil, nil)
 				}
@@ -1534,8 +1530,8 @@ func (h *fsmHandler) openconfirm(ctx context.Context) (bgp.FSMState, *fsmStateRe
 				h.conn.Close()
 				return bgp.BGP_FSM_IDLE, newfsmStateReason(fsmInvalidMsg, nil, nil)
 			case *bgp.MessageError:
-				m, _ := fsm.sendNotificationFromErrorMsg(e.MsgData.(*bgp.MessageError))
-				return bgp.BGP_FSM_IDLE, newfsmStateReason(fsmInvalidMsg, m, nil)
+				msg, _ := fsm.sendNotificationFromErrorMsg(m)
+				return bgp.BGP_FSM_IDLE, newfsmStateReason(fsmInvalidMsg, msg, nil)
 			default:
 				log.WithFields(log.Fields{
 					"Topic": "Peer",
